package it.openutils.configuration.services;

import it.openutils.configuration.dao.ReferenceCodeDAO;
import it.openutils.configuration.dao.ReferenceDomainDAO;
import it.openutils.configuration.dataobjects.ReferenceCode;
import it.openutils.configuration.dataobjects.ReferenceDomain;
import it.openutils.configuration.exceptions.ReferenceCodeSaveException;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;


/**
 * @author fgiust
 * @version $Id: ReferenceManagerImpl.java 179 2007-02-01 16:35:22Z fcarone $
 */
public class ReferenceManagerImpl implements ReferenceManager
{

    /**
     * Data access object for reference codes.
     */
    private ReferenceCodeDAO referenceCodeDAO;

    /**
     * Data access object for reference domains.
     */
    private ReferenceDomainDAO referenceDomainDAO;

    /**
     * IOC setter.
     * @param referenceCodeDAO the referenceCodeDAO to set
     */
    public void setReferenceCodeDAO(ReferenceCodeDAO referenceCodeDAO)
    {
        this.referenceCodeDAO = referenceCodeDAO;
    }

    /**
     * IOC setter.
     * @param referenceDomainDAO the referenceDomainDAO to set
     */
    public void setReferenceDomainDAO(ReferenceDomainDAO referenceDomainDAO)
    {
        this.referenceDomainDAO = referenceDomainDAO;
    }

    /**
     * {@inheritDoc}
     */
    public List<ReferenceDomain> getReferenceDomains()
    {
        return referenceDomainDAO.findAll();
    }


    /**
     * {@inheritDoc}
     */
    public ReferenceCode saveOrUpdateReferenceCode(ReferenceCode referenceCode)
    {
        ReferenceDomain transientDomain = referenceCode.getReferencedomain();
        if (transientDomain == null || (transientDomain.getId() == null))
        {
            throw new ReferenceCodeSaveException("Cannot save a ReferenceCode with an unknown ReferenceDomain.");
        }

        ReferenceCode filter = new ReferenceCode();
        ReferenceDomain dbDomain = referenceDomainDAO.loadIfAvailable(transientDomain.getId());
        if (dbDomain == null)
        {
            referenceDomainDAO.save(transientDomain);
        }
        else
        {
            filter.setReferencedomain(dbDomain);
        }

        filter.setCode(referenceCode.getCode());

        ReferenceCode dbCode = referenceCodeDAO.findFilteredFirst(filter);
        if (dbCode == null)
        {
            referenceCodeDAO.save(referenceCode);
            return referenceCode;
        }
        dbCode.setActive(referenceCode.getActive());
        dbCode.setCode(referenceCode.getCode());
        dbCode.setDescription(referenceCode.getDescription());
        dbCode.setReferencedomain(referenceCode.getReferencedomain());
        referenceCodeDAO.update(dbCode);
        return dbCode;
    }

    /**
     * {@inheritDoc}
     */
    public void removeReferenceCode(ReferenceCode referenceCode)
    {
        referenceCodeDAO.delete(referenceCode.getId());
    }

    /**
     * {@inheritDoc}
     */
    public List<ReferenceCode> getReferenceCodes(String domainId)
    {
        ReferenceDomain referenceDomain = referenceDomainDAO.loadIfAvailable(domainId);
        if (referenceDomain == null)
        {
            return new ArrayList<ReferenceCode>();
        }

        ReferenceCode filter = new ReferenceCode();
        filter.setReferencedomain(referenceDomain);

        List<ReferenceCode> referenceCodes = referenceCodeDAO.findFiltered(filter);
        Collections.sort(referenceCodes);
        return referenceCodes;
    }

    /**
     * {@inheritDoc}
     */
    public List<String> getReferenceCodesAsString(String domainId)
    {
        ReferenceDomain referenceDomain = referenceDomainDAO.loadIfAvailable(domainId);
        if (referenceDomain == null)
        {
            return new ArrayList<String>();
        }
        List<String> codeIds = new ArrayList<String>();

        ReferenceCode filter = new ReferenceCode();
        filter.setReferencedomain(referenceDomain);
        filter.setActive(Boolean.TRUE);

        List<ReferenceCode> codes = referenceCodeDAO.findFiltered(filter);

        for (ReferenceCode code : codes)
        {
            codeIds.add(code.getCode());
        }

        return codeIds;
    }

    /**
     * {@inheritDoc}
     */
    public ReferenceDomain getReferenceDomain(String domainId)
    {
        return referenceDomainDAO.load(domainId);
    }

    /**
     * {@inheritDoc}
     */
    public ReferenceDomain saveOrUpdateReferenceDomain(ReferenceDomain domain)
    {
        if (domain.getId() == null || referenceDomainDAO.loadIfAvailable(domain.getId()) == null)
        {
            referenceDomainDAO.save(domain);
            return domain;
        }
        ReferenceDomain dbDomain = referenceDomainDAO.loadIfAvailable(domain.getId());
        dbDomain.setDescription(domain.getDescription());
        referenceDomainDAO.update(dbDomain);
        return dbDomain;
    }

    /**
     * {@inheritDoc}
     */
    public Map<String, String> getReferenceCodesAsMap(String domainId)
    {
        List<ReferenceCode> codes = getReferenceCodes(domainId);
        Map<String, String> map = new LinkedHashMap<String, String>(codes.size());
        for (ReferenceCode code : codes)
        {
            if (code.getActive())
            {
                map.put(code.getCode(), code.getDescription());
            }
        }
        return map;
    }

    /**
     * {@inheritDoc}
     */
    public void removeReferenceCode(Long referenceCodeId)
    {
        referenceCodeDAO.delete(referenceCodeId);

    }

    /**
     * {@inheritDoc}
     */
    public ReferenceCode getReferenceCode(String domain, String code)
    {
        ReferenceCode filterCode = new ReferenceCode();
        filterCode.setCode(code);
        filterCode.setReferencedomain(getReferenceDomain(domain));
        return referenceCodeDAO.findFilteredFirst(filterCode);
    }

}
